@@ -17,7 +17,7 @@
 #include "konqhistorymanager.h"
 #include "konqpixmapprovider.h"
 #include "implementations/konqbrowserwindowinterface.h"
-#include "interfaces/downloaderextension.h"
+#include "interfaces/downloadjob.h"
 #include "browserarguments.h"
 #include "browserextension.h"
 #include "placeholderpart.h"
@@ -34,6 +34,7 @@
 #include <KUrlMimeData>
 #include <KIO/CopyJob>
 #include <KFileUtils>
+#include <KParts/ReadOnlyPart>
 
 #include <QApplication>
 #include <QArgument>
@@ -62,6 +63,8 @@
 
 //#define DEBUG_HISTORY
 
+using namespace Konq;
+
 KonqView::KonqView(KonqFrame *viewFrame, KonqMainWindow *mainWindow) :
     m_pPart{nullptr},
     m_pageSecurity{KonqMainWindow::NotCrypted},
@@ -97,7 +100,7 @@ KonqView::KonqView(KonqViewFactory &viewFactory,
                    const KPluginMetaData &service,
                    const QVector<KPluginMetaData> &partServiceOffers,
                    const KService::List &appServiceOffers,
-                   const QString &serviceType,
+                   const ViewType &serviceType,
                    bool passiveMode
                   )
 {
@@ -115,7 +118,7 @@ KonqView::KonqView(KonqViewFactory &viewFactory,
     m_service = service;
     m_partServiceOffers = partServiceOffers;
     m_appServiceOffers = appServiceOffers;
-    m_serviceType = serviceType;
+    m_type = serviceType;
 
     m_lstHistoryIndex = -1;
     m_bLoading = false;
@@ -161,9 +164,10 @@ KonqView::~KonqView()
     //qCDebug(KONQUEROR_LOG) << this << "done";
 }
 
-bool KonqView::isWebEngineView() const
+bool KonqView::isDevtoolsAvailable() const
 {
-    return m_service.pluginId() == QLatin1String("webenginepart");
+    static const auto inspectMethod = QMetaObject::normalizedSignature("setInspectedPart(KParts::ReadOnlyPart *)");
+    return m_pPart->metaObject()->indexOfMethod(inspectMethod.data()) != -1;
 }
 
 void KonqView::openUrl(const QUrl &url, const QString &locationBarURL,
@@ -178,7 +182,7 @@ void KonqView::openUrl(const QUrl &url, const QString &locationBarURL,
         args = m_pPart->arguments();
     }
 
-    KParts::NavigationExtension *ext = browserExtension();
+    KParts::NavigationExtension *ext = navigationExtension();
     BrowserArguments browserArgs;
     if (BrowserExtension *browserExtension = qobject_cast<BrowserExtension *>(ext)) {
         browserArgs = browserExtension->browserArguments();
@@ -186,7 +190,7 @@ void KonqView::openUrl(const QUrl &url, const QString &locationBarURL,
 
     // Typing "Enter" again after the URL of an aborted view, triggers a reload.
     if (m_bAborted && m_pPart && m_pPart->url() == url && !browserArgs.doPost()) {
-        if (!prepareReload(args, browserArgs, false /* not softReload */)) {
+        if (!prepareReload(args, browserArgs)) {
             return;
         }
         m_pPart->setArguments(args);
@@ -247,9 +251,6 @@ void KonqView::openUrl(const QUrl &url, const QString &locationBarURL,
     m_requestedUrl = !requestedUrl.isEmpty() ? requestedUrl : url;
     aboutToOpenURL(url, args);
 
-    if (args.metaData().contains("urlRequestedByApp") && isWebEngineView()) {
-        m_pPart->setProperty("urlRequestedByApp", url);
-    }
     m_pPart->openUrl(url);
 
     updateHistoryEntry(true);
@@ -279,7 +280,7 @@ void KonqView::switchViewMode(const QString& newPluginId, const QString& newInte
         const QUrl origUrl = realUrl();
         const QString locationBarURL = m_sLocationBarURL;
         bool tempFile = !m_tempFile.isEmpty();
-        changePart(serviceType(), newPluginId);
+        changePart(type(), newPluginId);
         openUrl(origUrl, locationBarURL, {}, tempFile, url());
     }
     if (changeViewMode){
@@ -295,7 +296,7 @@ void KonqView::switchView(KonqViewFactory &viewFactory, bool allowPlaceholder)
     if (!part) {
         return;
     }
-    
+
     m_pPart = part;
 
     // Set the statusbar in the BE asap to avoid a KMainWindow statusbar being created.
@@ -344,16 +345,17 @@ void KonqView::switchView(KonqViewFactory &viewFactory, bool allowPlaceholder)
 bool KonqView::ensureViewSupports(const QString &mimeType,
                                   bool forceAutoEmbed)
 {
+    ViewType type{mimeType};
     if (supportsMimeType(mimeType)) {
         // could be more specific, let's store it so that OpenUrlArguments::mimeType is correct
         // testcase: http://acid3.acidtests.org/svg.xml should be opened as image/svg+xml
-        m_serviceType = mimeType;
+        m_type = type;
         return true;
     }
-    return changePart(mimeType, QString(), forceAutoEmbed);
+    return changePart(type, QString(), forceAutoEmbed);
 }
 
-bool KonqView::changePart(const QString &mimeType,
+bool KonqView::changePart(const ViewType &type,
                           const QString &serviceName,
                           bool forceAutoEmbed)
 {
@@ -365,7 +367,7 @@ bool KonqView::changePart(const QString &mimeType,
     //             << "current service name=" << m_service->desktopEntryName();
 
     if (serviceName == m_service.pluginId()) {
-        m_serviceType = mimeType;
+        m_type = type;
         return true;
     }
 
@@ -378,20 +380,20 @@ bool KonqView::changePart(const QString &mimeType,
     KService::List appServiceOffers;
     KPluginMetaData service;
     KonqFactory konqFactory;
-    KonqViewFactory viewFactory = konqFactory.createView(mimeType, serviceName, &service, &partServiceOffers, &appServiceOffers, forceAutoEmbed);
+    KonqViewFactory viewFactory = konqFactory.createView(type, serviceName, &service, &partServiceOffers, &appServiceOffers, forceAutoEmbed);
 
     if (viewFactory.isNull()) {
         return false;
     }
 
-    m_serviceType = mimeType;
+    m_type = type;
     m_partServiceOffers = partServiceOffers;
     m_appServiceOffers = appServiceOffers;
 
     // Check if that's already the kind of part we have -> no need to recreate it
     // Note: we should have an operator== for KService...
     if (m_service.isValid() && m_service.pluginId() == service.pluginId()) {
-        qCDebug(KONQUEROR_LOG) << "Reusing service. Service type set to" << m_serviceType;
+        qCDebug(KONQUEROR_LOG) << "Reusing service. Service type set to" << m_type;
         if (m_pMainWindow->currentView() == this) {
             m_pMainWindow->updateViewModeActions();
         }
@@ -422,29 +424,18 @@ void KonqView::connectPart()
                 m_pMainWindow, SLOT(slotInternalViewModeChanged()));
     }
 
-    if (m_pPart) {
-        KonqInterfaces::DownloaderExtension *dext = KonqInterfaces::DownloaderExtension::downloader(m_pPart);
-        if (dext) {
-            connect(dext, &KonqInterfaces::DownloaderExtension::downloadAndOpenUrl, m_pMainWindow, [dext, this](const QUrl &url, int id, const KParts::OpenUrlArguments &args, const BrowserArguments &bargs, bool temp){
-                    KonqOpenURLRequest req(args, bargs, dext->part(), true, id);
-                    req.tempFile = temp;
-                    m_pMainWindow->slotOpenURLRequest(url, req);
-                });
-        }
-    }
-
-    KParts::NavigationExtension *ext = browserExtension();
+    KParts::NavigationExtension *ext = navigationExtension();
     if (ext) {
-
         if (auto browserExtension = qobject_cast<BrowserExtension *>(ext)) {
-            connect(browserExtension, &BrowserExtension::browserOpenUrlRequestDelayed, m_pMainWindow, [ext, this](const QUrl &url, const KParts::OpenUrlArguments &arguments, const BrowserArguments &browserArguments){
+            connect(browserExtension, &BrowserExtension::browserOpenUrlRequestDelayed, m_pMainWindow, [ext, this](const QUrl &url, const KParts::OpenUrlArguments &arguments, const BrowserArguments &browserArguments, bool temp){
                 KonqOpenURLRequest req(arguments, browserArguments, qobject_cast<KParts::ReadOnlyPart*>(ext->parent()));
-                    m_pMainWindow->slotOpenURLRequest(url, req);
+                req.tempFile = temp;
+                m_pMainWindow->slotOpenURLRequest(url, req);
             });
             KonqBrowserWindowInterface *bi = new KonqBrowserWindowInterface(mainWindow(), m_pPart);
             browserExtension->setBrowserInterface(bi);
         } else {
-            connect(ext, &BrowserExtension::openUrlRequestDelayed, m_pMainWindow,
+            connect(ext, &KParts::NavigationExtension::openUrlRequestDelayed, m_pMainWindow,
                 [ext, this](const QUrl &url, const KParts::OpenUrlArguments &args){
                     KonqOpenURLRequest req(args, {}, qobject_cast<KParts::ReadOnlyPart*>(ext->parent()));
                     m_pMainWindow->slotOpenURLRequest(url, req);
@@ -827,11 +818,11 @@ void KonqView::updateHistoryEntry(bool needsReload)
 #endif
 
     current->reload = needsReload; // We have a state for it now.
-    if (!needsReload && browserExtension()) {
+    if (!needsReload && navigationExtension()) {
         current->buffer = QByteArray(); // Start with empty buffer.
         QDataStream stream(&current->buffer, QIODevice::WriteOnly);
 
-        browserExtension()->saveState(stream);
+        navigationExtension()->saveState(stream);
     }
 
 #ifdef DEBUG_HISTORY
@@ -850,7 +841,7 @@ void KonqView::updateHistoryEntry(bool needsReload)
     qCDebug(KONQUEROR_LOG) << "Saving title:" << m_caption << "in history position" << historyIndex();
 #endif
     current->title = m_caption;
-    current->strServiceType = m_serviceType;
+    current->strViewType = m_type.toString();
     current->strServiceName = m_service.pluginId();
 
     current->doPost = m_doPost;
@@ -904,8 +895,8 @@ void KonqView::restoreHistory()
     setPageSecurity(h.pageSecurity);
     m_sTypedURL.clear();
 
-    if (!changePart(h.strServiceType, h.strServiceName)) {
-        qCWarning(KONQUEROR_LOG) << "Couldn't change view mode to" << h.strServiceType << h.strServiceName;
+    if (!changePart(ViewType::fromString(h.strViewType), h.strServiceName)) {
+        qCWarning(KONQUEROR_LOG) << "Couldn't change view mode to" << h.strViewType << h.strServiceName;
         return /*false*/;
     }
 
@@ -914,11 +905,11 @@ void KonqView::restoreHistory()
     m_requestedUrl = h.url;
     aboutToOpenURL(h.url);
 
-    if (h.reload == false && browserExtension() && historyIndex() > 0) {
+    if (h.reload == false && navigationExtension() && historyIndex() > 0) {
         //qCDebug(KONQUEROR_LOG) << "Restoring view from stream";
         QDataStream stream(h.buffer);
 
-        browserExtension()->restoreState(stream);
+        navigationExtension()->restoreState(stream);
 
         m_doPost = h.doPost;
         m_postContentType = h.postContentType;
@@ -1082,7 +1073,7 @@ void KonqView::aboutToOpenURL(const QUrl &url, const KParts::OpenUrlArguments &a
 void KonqView::setPartMimeType()
 {
     KParts::OpenUrlArguments args(m_pPart->arguments());
-    args.setMimeType(m_serviceType);
+    args.setMimeType(m_type.isMimetype() ? m_type.mimetype().value() : QString());
     m_pPart->setArguments(args);
 }
 
@@ -1133,7 +1124,7 @@ void KonqView::enablePopupMenu(bool b)
 {
     Q_ASSERT(m_pMainWindow);
 
-    KParts::NavigationExtension *ext = browserExtension();
+    KParts::NavigationExtension *ext = navigationExtension();
 
     if (!ext) {
         return;
@@ -1263,7 +1254,7 @@ bool KonqView::eventFilter(QObject *obj, QEvent *e)
         const QMimeData *mimeData = ev->mimeData();
 
         QList<QUrl> lstDragURLs = KUrlMimeData::urlsFromMimeData(mimeData);
-        KParts::NavigationExtension *ext = browserExtension();
+        KParts::NavigationExtension *ext = navigationExtension();
         if (!lstDragURLs.isEmpty() && ext && lstDragURLs.first().isValid()) {
             emit ext->openUrlRequest(lstDragURLs.first());    // this will call m_pMainWindow::slotOpenURLRequest delayed
         }
@@ -1271,12 +1262,9 @@ bool KonqView::eventFilter(QObject *obj, QEvent *e)
     return false;
 }
 
-bool KonqView::prepareReload(KParts::OpenUrlArguments &args, BrowserArguments &browserArgs, bool softReload)
+bool KonqView::prepareReload(KParts::OpenUrlArguments &args, BrowserArguments &browserArgs)
 {
     args.setReload(true);
-    if (softReload) {
-        browserArgs.softReload = true;
-    }
 
     // Repost form data if this URL is the result of a POST HTML form.
     if (m_doPost && !browserArgs.redirectedRequest()) {
@@ -1297,7 +1285,7 @@ bool KonqView::prepareReload(KParts::OpenUrlArguments &args, BrowserArguments &b
     return true;
 }
 
-KParts::NavigationExtension *KonqView::browserExtension() const
+KParts::NavigationExtension *KonqView::navigationExtension() const
 {
     return m_pPart ? KParts::NavigationExtension::childObject(m_pPart) : nullptr;
 }
@@ -1309,8 +1297,11 @@ KParts::StatusBarExtension *KonqView::statusBarExtension() const
 
 QMimeType KonqView::mimeType() const
 {
+    if (m_type.isCapability()) {
+        return {};
+    }
     QMimeDatabase db;
-    return db.mimeTypeForName(serviceType());
+    return db.mimeTypeForName(type().mimetype().value());
 }
 
 bool KonqView::supportsMimeType(const QString &mimeType) const
@@ -1325,23 +1316,13 @@ bool KonqView::supportsMimeType(const QString &mimeType) const
     return found != lst.constEnd();
 }
 
-bool KonqView::isWebBrowsingPart() const
-{
-    if (!m_pPart) {
-        return false;
-    }
-    const QString partName = m_pPart->componentName();
-    return partName == QLatin1String("webenginepart") || partName == QLatin1String("khtml") || partName == QLatin1String("kwebkitpart");
-}
-
-
 void HistoryEntry::saveConfig(KConfigGroup &config, const QString &prefix, const KonqFrameBase::Options &options)
 {
     if (options & KonqFrameBase::SaveUrls) {
         config.writeEntry(QStringLiteral("Url").prepend(prefix), url.url());
         config.writeEntry(QStringLiteral("LocationBarURL").prepend(prefix), locationBarURL);
         config.writeEntry(QStringLiteral("Title").prepend(prefix), title);
-        config.writeEntry(QStringLiteral("StrServiceType").prepend(prefix), strServiceType);
+        config.writeEntry(QStringLiteral("StrServiceType").prepend(prefix), strViewType);
         config.writeEntry(QStringLiteral("StrServiceName").prepend(prefix), strServiceName);
     } else if (options & KonqFrameBase::SaveHistoryItems) {
         config.writeEntry(QStringLiteral("Url").prepend(prefix), url.url());
@@ -1349,7 +1330,7 @@ void HistoryEntry::saveConfig(KConfigGroup &config, const QString &prefix, const
         config.writeEntry(QStringLiteral("LocationBarURL").prepend(prefix), locationBarURL);
         config.writeEntry(QStringLiteral("Title").prepend(prefix), title);
         config.writeEntry(QStringLiteral("Buffer").prepend(prefix), buffer);
-        config.writeEntry(QStringLiteral("StrServiceType").prepend(prefix), strServiceType);
+        config.writeEntry(QStringLiteral("StrServiceType").prepend(prefix), strViewType);
         config.writeEntry(QStringLiteral("StrServiceName").prepend(prefix), strServiceName);
         config.writeEntry(QStringLiteral("PostData").prepend(prefix), postData);
         config.writeEntry(QStringLiteral("PostContentType").prepend(prefix), postContentType);
@@ -1366,7 +1347,7 @@ HistoryEntry* HistoryEntry::fromDelayedLoadingData(const KConfigGroup& config, c
         entry->url = QUrl(config.readEntry(QStringLiteral("Url").prepend(prefix), ""));
         entry->locationBarURL = config.readEntry(QStringLiteral("LocationBarURL").prepend(prefix), "");
         entry->title = config.readEntry(QStringLiteral("Title").prepend(prefix), "");
-        entry->strServiceType = config.readEntry(QStringLiteral("StrServiceType").prepend(prefix), "");
+        entry->strViewType = config.readEntry(QStringLiteral("StrServiceType").prepend(prefix), "");
         entry->strServiceName = config.readEntry(QStringLiteral("StrServiceName").prepend(prefix), "");
     }
     if (options & KonqFrameBase::SaveUrls) {
@@ -1395,11 +1376,11 @@ void KonqView::saveConfig(KConfigGroup &config, const QString &prefix, const Kon
     //This can happen when loading a session with several tabs, then saving it again without activating all the
     //tabs
     Konq::PlaceholderPart *plPart = placeholderPart();
-    QString mimeType = plPart ? plPart->delayedLoadingData().mimeType : serviceType();
+    QString viewType = (plPart ? plPart->delayedLoadingData().type : type()).toString();
     QString serviceName = plPart ? plPart->delayedLoadingData().serviceName : service().pluginId();
     bool locked = plPart ? plPart->delayedLoadingData().lockedLocation : isLockedLocation();
 
-    config.writeEntry(QStringLiteral("ServiceType").prepend(prefix), mimeType);
+    config.writeEntry(QStringLiteral("ServiceType").prepend(prefix), viewType);
     config.writeEntry(QStringLiteral("ServiceName").prepend(prefix), serviceName);
     config.writeEntry(QStringLiteral("PassiveMode").prepend(prefix), isPassiveMode());
     config.writeEntry(QStringLiteral("LinkedView").prepend(prefix), isLinkedView());
@@ -1428,13 +1409,13 @@ void KonqView::saveConfig(KConfigGroup &config, const QString &prefix, const Kon
     }
 }
 
-void KonqView::storeDelayedLoadingData(const QString& mimeType, const QString& serviceName, bool openUrl, const QUrl& url, bool lockedLocation, const KConfigGroup& grp, const QString& prefix)
+void KonqView::storeDelayedLoadingData(const ViewType& type, const QString& serviceName, bool openUrl, const QUrl& url, bool lockedLocation, const KConfigGroup& grp, const QString& prefix)
 {
     Konq::PlaceholderPart *plPart = placeholderPart();
     if (!plPart) {
         return;
     }
-    plPart->setDelayedLoadingData({mimeType, serviceName, openUrl, url, lockedLocation});
+    plPart->setDelayedLoadingData({type, serviceName, openUrl, url, lockedLocation});
 
     const QString keyHistoryItems = QStringLiteral("NumberOfHistoryItems").prepend(prefix);
 
@@ -1468,7 +1449,7 @@ void KonqView::loadDelayed()
     }
 
     Konq::PlaceholderPart::DelayedLoadingData data = plPart->delayedLoadingData();
-    changePart(data.mimeType, data.serviceName, true);
+    changePart({data.type}, data.serviceName, true);
 
     if (data.openUrl) {
         if (m_lstHistory.isEmpty()) {
@@ -1477,7 +1458,7 @@ void KonqView::loadDelayed()
             if (!KonqUrl::hasKonqScheme(url)) {
                 req.typedUrl = url.toDisplayString();
             }
-            m_pMainWindow->openView(data.mimeType, url, this, req);
+            m_pMainWindow->openView(data.type, url, this, req);
         } else {
             restoreHistory();
         }
